/*****************************************************************************\
* (c) Copyright 2000-2021 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// Class: ReadMLPHInner

/* configuration options =====================================================

#GEN -*-*-*-*-*-*-*-*-*-*-*- general info -*-*-*-*-*-*-*-*-*-*-*-

Method         : MLP::MLP
TMVA Release   : 4.2.1         [262657]
ROOT Release   : 6.24/00       [399360]
Creator        : calvom
Date           : Thu Jul  1 15:58:11 2021
Host           : Linux centos7-docker 4.18.0-193.19.1.el8_2.x86_64 #1 SMP Mon Sep 14 14:37:00 UTC 2020 x86_64 x86_64
x86_64 GNU/Linux Training events: 194000 Analysis type  : [Classification]

#OPT -*-*-*-*-*-*-*-*-*-*-*-*- options -*-*-*-*-*-*-*-*-*-*-*-*-

# Set by User:
NCycles: "500" [Number of training cycles]
HiddenLayers: "N" [Specification of hidden layer architecture]
NeuronType: "tanh" [Neuron activation function type]
V: "False" [Verbose output (short form of "VerbosityLevel" below - overrides the latter one)]
VarTransform: "N" [List of variable transformations performed before training, e.g.,
"D_Background,P_Signal,G,N_AllClasses" for: "Decorrelation, PCA-transformation, Gaussianisation, Normalisation, each for
the given class of events ('Al$ H: "True" [Print method-specific help message] TestRate: "5" [Test for overtraining
performed at each #th epochs] UseRegulator: "False" [Use regulator to avoid over-training] # Default: RandomSeed: "1"
[Random seed for initial synapse weights (0 means unique seed for each run; default value '1')] EstimatorType: "CE" [MSE
(Mean Square Estimator) for Gaussian Likelihood or CE(Cross-Entropy) for Bernoulli Likelihood] NeuronInputType: "sum"
[Neuron input function type] VerbosityLevel: "Default" [Verbosity level] CreateMVAPdfs: "False" [Create PDFs for
classifier outputs (signal and background)] IgnoreNegWeightsInTraining: "False" [Events with negative weights are
ignored in the training (but are included for testing and performance evaluation)] TrainingMethod: "BP" [Train with
Back-Propagation (BP), BFGS Algorithm (BFGS), or Genetic Algorithm (GA - slower and worse)] LearningRate: "2.000000e-02"
[ANN learning rate parameter] DecayRate: "1.000000e-02" [Decay rate for learning parameter] EpochMonitoring: "False"
[Provide epoch-wise monitoring plots according to TestRate (caution: causes big ROOT output file!)] Sampling:
"1.000000e+00" [Only 'Sampling' (randomly selected) events are trained each epoch] SamplingEpoch: "1.000000e+00"
[Sampling is used for the first 'SamplingEpoch' epochs, afterwards, all events are taken for training]
SamplingImportance: "1.000000e+00" [ The sampling weights of events in epochs which successful (worse estimator than
before) are multiplied with SamplingImportance, else they are divided.] SamplingTraining: "True" [The training sample is
sampled] SamplingTesting: "False" [The testing sample is sampled] ResetStep: "50" [How often BFGS should reset history]
Tau: "3.000000e+00" [LineSearch "size step"]
BPMode: "sequential" [Back-propagation learning mode: sequential or batch]
BatchSize: "-1" [Batch size: number of events/batch, only set if in Batch Mode, -1 for BatchSize=number_of_events]
ConvergenceImprove: "1.000000e-30" [Minimum improvement which counts as improvement (<0 means automatic convergence
check is turned off)] ConvergenceTests: "-1" [Number of steps (without improvement) required for convergence (<0 means
automatic convergence check is turned off)] UpdateLimit: "10000" [Maximum times of regulator update] CalculateErrors:
"False" [Calculates inverse Hessian matrix at the end of the training to be able to calculate the uncertainties of an
MVA value] WeightRange: "1.000000e+00" [Take the events for the estimator calculations from small deviations from the
desired value to large deviations only over the weight range]
##


#VAR -*-*-*-*-*-*-*-*-*-*-*-* variables *-*-*-*-*-*-*-*-*-*-*-*-

NVar 9
fracE1                  fracE1                fracE1                  fracE1 'F'
[-0.140813097358,0.44899430871] fracE2                  fracE2                fracE2 fracE2
'F'    [-0.139410734177,0.539303302765] fracE3                  fracE3                fracE3
fracE3                                                    'F'    [-0.137271896005,0.482935339212] fracE4
fracE4               fracE4                 fracE4 'F'    [-0.147227719426,0.502812743187]
fracEseed                 fracEseed               fracEseed                 fracEseed 'F'
[0.135082438588,1.31137573719] fracE6                 fracE6               fracE6 fracE6
'F'    [-0.160523056984,0.519417881966] fracE7                 fracE7               fracE7
fracE7                                                   'F'    [-0.195828691125,0.44386869669] fracE8
fracE8               fracE8                 fracE8 'F'    [-0.153247758746,0.533139109612]
fracE9                 fracE9               fracE9                 fracE9 'F'
[-0.172588512301,0.459551632404] NSpec 0

============================================================================ */

#include "Kernel/STLExtensions.h"
#include "Kernel/TMV_utils.h"
#include <array>
#include <cmath>
#include <string_view>

namespace Data::ReadMLPHInner {
  namespace {
    constexpr auto ActivationFnc = []( double x ) {
      // fast hyperbolic tan approximation
      if ( x > 4.97 ) return 1.f;
      if ( x < -4.97 ) return -1.f;
      float x2 = x * x;
      float a  = x * ( 135135.0f + x2 * ( 17325.0f + x2 * ( 378.0f + x2 ) ) );
      float b  = 135135.0f + x2 * ( 62370.0f + x2 * ( 3150.0f + x2 * 28.0f ) );
      return a / b;
    };

    constexpr auto OutputActivationFnc = []( double x ) {
      // sigmoid
      return 1.0 / ( 1.0 + exp( -x ) );
    };

    // Normalization transformation
    constexpr auto fMin_1 =
        std::array{ std::array{ -0.126388713717, -0.139410734177, -0.137271896005, -0.147227719426, 0.200036898255,
                                -0.128019049764, -0.195828691125, -0.114431194961, -0.172588512301 },
                    std::array{ -0.140813097358, -0.11138343811, -0.125993549824, -0.144452929497, 0.135082438588,
                                -0.160523056984, -0.131005212665, -0.153247758746, -0.137483417988 },
                    std::array{ -0.140813097358, -0.139410734177, -0.137271896005, -0.147227719426, 0.135082438588,
                                -0.160523056984, -0.195828691125, -0.153247758746, -0.172588512301 } };

    constexpr auto fMax_1 =
        std::array{ std::array{ 0.352079391479, 0.539303302765, 0.449519902468, 0.491340994835, 1.31137573719,
                                0.519417881966, 0.399017363787, 0.533139109612, 0.447410911322 },
                    std::array{ 0.44899430871, 0.468828141689, 0.482935339212, 0.502812743187, 1.19125139713,
                                0.485027074814, 0.44386869669, 0.499527543783, 0.459551632404 },
                    std::array{ 0.44899430871, 0.539303302765, 0.482935339212, 0.502812743187, 1.31137573719,
                                0.519417881966, 0.44386869669, 0.533139109612, 0.459551632404 } };

    // weight matrix from layer 0 to 1
    constexpr auto fWeightMatrix0to1 = std::array{
        std::array{ -0.768344824855872, -2.1979013688362, -0.939913092928029, -0.906947937490384, -1.70515506237624,
                    -0.319068335017765, 0.299355753981265, 3.31142967799943, 0.223340960415213, -0.212439754995487 },
        std::array{ 1.80608594199172, -2.50530556844779, -0.637959055875342, -2.96764335380513, 4.10491609257577,
                    1.87106057184041, -0.635751711435271, 1.85526375228023, 1.57555573146871, 0.370502353568081 },
        std::array{ 0.527248125813546, -0.416913102861955, 0.192125732159274, 1.4445082384563, 0.604861550288342,
                    -3.30117781609398, 0.689407636883195, 0.685505265981109, -0.984558296619742, -1.42872690857851 },
        std::array{ 5.33831543630887, 5.17245423720476, 5.26776637871476, 6.16370813234337, 10.9801192075486,
                    6.41712694647997, -3.73400173491101, -24.7275966326113, -2.84140805244608, -5.02468155365119 },
        std::array{ 4.48669917474464, -1.8487540552227, 0.613701892156025, -1.78023054865855, 0.327403107109213,
                    -0.666227954795512, 0.974388375761939, -0.353049990455244, -2.77740820742302, 0.820874783902317 },
        std::array{ -1.69847179684072, -0.175365363605958, -0.829464692680845, 0.0046316076121839, 1.92780784733841,
                    0.657651966900329, -0.51442592136225, 0.683821680521208, 0.451331341128691, 0.648010448886695 },
        std::array{ 1.81895282731932, 2.17672682658848, -0.920765353140827, 0.996747884904092, 1.09653368948432,
                    -1.67679120435122, -0.140763100862611, -1.75879692246824, 0.740947618892347, -1.427175677997 },
        std::array{ 7.0573188080443, 7.8451092078208, -6.63073158246502, 6.95138315493782, 14.016929766438,
                    -28.0199051775889, 6.89190760271404, 8.24592723355141, -6.16184568139477, -7.34051531829907 },
        std::array{ -0.0733444989313001, 1.38406746094216, 1.45192278216375, -1.16397395366091, 0.0670610984506287,
                    1.6294400538039, 1.30327927516488, -2.16140295353957, -1.06572937188725, -1.57791855454015 } };
    // weight matrix from layer 1 to 2
    constexpr auto fWeightMatrix1to2 =
        std::array{ -2.68994429564918, 1.58978773372041, 3.91993950979052,  -1.5535261985431, 1.19928779393608,
                    4.65746096444488,  3.26594013318893, -1.70881031348903, 2.76660022788495, 3.77460699206932 };

    // the training input variables
    constexpr auto validator =
        TMV::Utils::Validator{ "ReadMLPHInner", std::tuple{ "fracE1", "fracE2", "fracE3", "fracE4", "fracEseed",
                                                            "fracE6", "fracE7", "fracE8", "fracE9" } };

    constexpr auto transformer = TMV::Utils::Transformer{ fMin_1, fMax_1 };
    constexpr auto l0To1       = TMV::Utils::Layer{ fWeightMatrix0to1, ActivationFnc };
    constexpr auto l1To2       = TMV::Utils::Layer{ fWeightMatrix1to2, OutputActivationFnc };
    constexpr auto MVA         = TMV::Utils::MVA{ validator, transformer, 2, l0To1, l1To2 };
  } // namespace
} // namespace Data::ReadMLPHInner

struct ReadMLPHInner final {

  // constructor
  ReadMLPHInner( LHCb::span<const std::string_view, 9> theInputVars ) {
    Data::ReadMLPHInner::MVA.validate( theInputVars );
  }

  // the classifier response
  // "inputValues" is a vector of input values in the same order as the
  // variables given to the constructor
  static constexpr double GetMvaValue( LHCb::span<const double, 9> input ) { return Data::ReadMLPHInner::MVA( input ); }
};
