/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef IDAVINCIASSOCIATORSWRAPPER_H
#define IDAVINCIASSOCIATORSWRAPPER_H 1

// Include files
// from STL
#include <string>

// from Gaudi
#include "GaudiKernel/IAlgTool.h"

#include "Kernel/Particle2MCLinker.h"
#include "Kernel/Particle2MCMethod.h"

static const InterfaceID IID_IDaVinciAssociatorsWrapper( "IDaVinciAssociatorsWrapper", 1, 0 );

/** @class IDaVinciAssociatorsWrapper IDaVinciAssociatorsWrapper.h
 *
 *  Wrapper around  Particle2MCLinker class
 *
 *  To use:
 *
 *  Object2FromMC<LHCb::Particle, GaudiTool> mylinker = m_myLinkerTool->linker(linkerMethod,container);
 *
 *  where linker Method corresponds to one of the Methods declared in Kernel/Particle2MCMethod.h,
 *  for example Link, Composite, Chi2. Container allows you to specify a specifi location for the
 *  particles to be associated, It defaults to 0 if omitted:
 *
 *  Object2FromMC<LHCb::Particle, GaudiTool> mylinker = m_myLinkerTool->linker(linkerMethod);
 *
 *  Note that you will need to declare a different instance of this tool
 *  for each linker which you wish to use. Declare an instance in your
 *  algorithm's initialize:
 *
 *  m_myLinkerTool = tool<IDaVinciAssociatorsWrapper>("DaVinciAssociatorsWrapper","exampleTool",this);
 *
 *  @author V. Gligorov (based on code by P. Koppenburg)
 *  @date   2008-06-26
 */
class IDaVinciAssociatorsWrapper : virtual public IAlgTool {
public:
  // Return the interface ID
  static const InterfaceID& interfaceID() { return IID_IDaVinciAssociatorsWrapper; }

  virtual Object2FromMC<LHCb::Particle, GaudiTool>*
  linker( const Particle2MCMethod::AssociationMethod& myMethod,
          const std::vector<std::string>& myLocations = std::vector<std::string>( 1, "" ) ) = 0; ///< returns a linker
};
#endif // IDAVINCIASSOCIATORSWRAPPER_H
