/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef CONE_INFO_H
#define CONE_INFO_H 1

#include "Event/RelatedInfoMap.h"

class RelatedInfoNamed : public LHCb::RelatedInfoMap {

public:
  /// Index values
  enum index {
    UNKNOWN = -1, // Unknown/illegal value of the index

    // Cone variables
    CONEANGLE    = 0,              // Cone angle
    CONEMULT     = CONEANGLE + 1,  // Cone multiplicity
    CONEPX       = CONEANGLE + 2,  // Cone Px
    CONEPY       = CONEANGLE + 3,  // Cone Py
    CONEPZ       = CONEANGLE + 4,  // Cone Pz
    CONEP        = CONEANGLE + 5,  // Cone P
    CONEPT       = CONEANGLE + 6,  // Cone Pt
    CONEPXASYM   = CONEANGLE + 7,  // Cone Px asymmetry
    CONEPYASYM   = CONEANGLE + 8,  // Cone Py asymmetry
    CONEPZASYM   = CONEANGLE + 9,  // Cone Pz asymmetry
    CONEPASYM    = CONEANGLE + 10, // Cone P asymmetry
    CONEPTASYM   = CONEANGLE + 11, // Cone Pt asymmetry
    CONEDELTAETA = CONEANGLE + 12, // Cone DeltaEta
    CONEDELTAPHI = CONEANGLE + 13, // Cone DeltaPhi

    // Cone variables for EW
    EWCONEANGLE   = CONEDELTAPHI + 1, // Cone angle
    EWCONEMULT    = EWCONEANGLE + 1,  // Cone multiplicity
    EWCONEPX      = EWCONEANGLE + 2,  // Cone Px
    EWCONEPY      = EWCONEANGLE + 3,  // Cone Py
    EWCONEPZ      = EWCONEANGLE + 4,  // Cone Pz
    EWCONEVP      = EWCONEANGLE + 5,  // Cone vP
    EWCONEVPT     = EWCONEANGLE + 6,  // Cone vPt
    EWCONESP      = EWCONEANGLE + 7,  // Cone sP
    EWCONESPT     = EWCONEANGLE + 8,  // Cone sPt
    EWCONETP      = EWCONEANGLE + 9,  // Cone tP
    EWCONETPT     = EWCONEANGLE + 10, // Cone tPt
    EWCONEMINPTE  = EWCONEANGLE + 11, // Cone min(Pt_e)
    EWCONEMAXPTE  = EWCONEANGLE + 12, // Cone max(Pt_e)
    EWCONEMINPTMU = EWCONEANGLE + 13, // Cone min(Pt_mu)
    EWCONEMAXPTMU = EWCONEANGLE + 14, // Cone max(Pt_mu)
    EWCONENMULT   = EWCONEANGLE + 15, // Neutral cone multiplicity
    EWCONENPX     = EWCONEANGLE + 16, // Neutral Cone Px
    EWCONENPY     = EWCONEANGLE + 17, // Neutral Cone Py
    EWCONENPZ     = EWCONEANGLE + 18, // Neutral Cone Pz
    EWCONENVP     = EWCONEANGLE + 19, // Neutral Cone vP
    EWCONENVPT    = EWCONEANGLE + 20, // Neutral Cone vPt
    EWCONENSP     = EWCONEANGLE + 21, // Neutral Cone sP
    EWCONENSPT    = EWCONEANGLE + 22, // Neutral Cone sPt

    // Vertex isolation variables
    VTXISONUMVTX            = EWCONENSPT + 1,   // NumVtxWithinChi2WindowOneTrack
    VTXISODCHI2ONETRACK     = VTXISONUMVTX + 1, // SmallestDeltaChi2OneTrack
    VTXISODCHI2MASSONETRACK = VTXISONUMVTX + 2, // SmallestDeltaChi2MassOneTrack
    VTXISODCHI2TWOTRACK     = VTXISONUMVTX + 3, // SmallestDeltaChi2OneTrack
    VTXISODCHI2MASSTWOTRACK = VTXISONUMVTX + 4, // SmallestDeltaChi2MassOneTrack

    // BDT ISOLATIONS ENUMS
    VTXISOBDTHARDFIRSTVALUE  = VTXISODCHI2MASSTWOTRACK + 1,
    VTXISOBDTHARDSECONDVALUE = VTXISOBDTHARDFIRSTVALUE + 1,
    VTXISOBDTHARDTHIRDVALUE  = VTXISOBDTHARDFIRSTVALUE + 2,
    VTXISOBDTSOFTFIRSTVALUE  = VTXISOBDTHARDFIRSTVALUE + 3,
    VTXISOBDTSOFTSECONDVALUE = VTXISOBDTHARDFIRSTVALUE + 4,
    VTXISOBDTSOFTTHIRDVALUE  = VTXISOBDTHARDFIRSTVALUE + 5,
    TRKISOBDTFIRSTVALUE      = VTXISOBDTHARDFIRSTVALUE + 6,
    TRKISOBDTSECONDVALUE     = VTXISOBDTHARDFIRSTVALUE + 7,
    TRKISOBDTTHIRDVALUE      = VTXISOBDTHARDFIRSTVALUE + 8,

    BSMUMUCDFISO           = TRKISOBDTTHIRDVALUE + 1,
    BSMUMUOTHERBMAG        = BSMUMUCDFISO + 1,
    BSMUMUOTHERBANGLE      = BSMUMUCDFISO + 2,
    BSMUMUOTHERBBOOSTMAG   = BSMUMUCDFISO + 3,
    BSMUMUOTHERBBOOSTANGLE = BSMUMUCDFISO + 4,
    BSMUMUTRACKPLUSISO     = BSMUMUCDFISO + 5,
    BSMUMUTRACKMINUSISO    = BSMUMUCDFISO + 6,
    BSMUMUOTHERBTRACKS     = BSMUMUCDFISO + 7,

    // Jet isolation variables
    JETNOMU1PX       = BSMUMUOTHERBTRACKS + 1,
    JETNOMU1PY       = JETNOMU1PX + 1,
    JETNOMU1PZ       = JETNOMU1PX + 2,
    JETNOMU1PT       = JETNOMU1PX + 3,
    JETNOMU1JETWIDTH = JETNOMU1PX + 4,
    JETNOMU1NNTAG    = JETNOMU1PX + 5,
    JETNOMU1MNF      = JETNOMU1PX + 6,
    JETNOMU2PX       = JETNOMU1PX + 7,
    JETNOMU2PY       = JETNOMU1PX + 8,
    JETNOMU2PZ       = JETNOMU1PX + 9,
    JETNOMU2PT       = JETNOMU1PX + 10,
    JETNOMU2JETWIDTH = JETNOMU1PX + 11,
    JETNOMU2NNTAG    = JETNOMU1PX + 12,
    JETNOMU2MNF      = JETNOMU1PX + 13,
    JETNOMU3PX       = JETNOMU1PX + 14,
    JETNOMU3PY       = JETNOMU1PX + 15,
    JETNOMU3PZ       = JETNOMU1PX + 16,
    JETNOMU3PT       = JETNOMU1PX + 17,
    JETNOMU3JETWIDTH = JETNOMU1PX + 18,
    JETNOMU3NNTAG    = JETNOMU1PX + 19,
    JETNOMU3MNF      = JETNOMU1PX + 20,
    JETMU1PX         = JETNOMU1PX + 21,
    JETMU1PY         = JETNOMU1PX + 22,
    JETMU1PZ         = JETNOMU1PX + 23,
    JETMU1PT         = JETNOMU1PX + 24,
    JETMU1JETWIDTH   = JETNOMU1PX + 25,
    JETMU1NNTAG      = JETNOMU1PX + 26,
    JETMU1MNF        = JETNOMU1PX + 27,
    JETMU2PX         = JETNOMU1PX + 28,
    JETMU2PY         = JETNOMU1PX + 29,
    JETMU2PZ         = JETNOMU1PX + 30,
    JETMU2PT         = JETNOMU1PX + 31,
    JETMU2JETWIDTH   = JETNOMU1PX + 32,
    JETMU2NNTAG      = JETNOMU1PX + 33,
    JETMU2MNF        = JETNOMU1PX + 34,
    JETBPX           = JETNOMU1PX + 35,
    JETBPY           = JETNOMU1PX + 36,
    JETBPZ           = JETNOMU1PX + 37,
    JETBPT           = JETNOMU1PX + 38,
    JETBJETWIDTH     = JETNOMU1PX + 39,
    JETBNNTAG        = JETNOMU1PX + 40,
    JETBMNF          = JETNOMU1PX + 41,

    // cylinder isolation variables
    CYLBESTD0              = JETBMNF + 1,
    CYLBESTKS              = CYLBESTD0 + 1,
    CYLTRACKSNCYL          = CYLBESTD0 + 2,
    CYLMINPT               = CYLBESTD0 + 3,
    CYLMAXPT               = CYLBESTD0 + 4,
    CYLAVPT                = CYLBESTD0 + 5,
    CYLMAXPTTRIPCHI2       = CYLBESTD0 + 6,
    CYLMAXPTTRDOCACHI2     = CYLBESTD0 + 7,
    CYLMAXPTTRBMASS        = CYLBESTD0 + 8,
    CYLMINIPCHI2           = CYLBESTD0 + 9,
    CYLMAXIPCHI2           = CYLBESTD0 + 10,
    CYLAVIPCHI2            = CYLBESTD0 + 11,
    CYLMAXIPCHI2TRPT       = CYLBESTD0 + 12,
    CYLMAXIPCHI2TRDOCACHI2 = CYLBESTD0 + 13,
    CYLMAXIPCHI2TRBMASS    = CYLBESTD0 + 14,
    CYLMINDOCACHI2         = CYLBESTD0 + 15,
    CYLMAXDOCACHI2         = CYLBESTD0 + 16,
    CYLAVDOCACHI2          = CYLBESTD0 + 17,
    CYLMINDOCATRPT         = CYLBESTD0 + 18,
    CYLMINDOCATRIPCHI2     = CYLBESTD0 + 19,
    CYLMINDOCATRBMASS      = CYLBESTD0 + 20,
    CYLBESTBMASS           = CYLBESTD0 + 21,
    CYLBESTBMASSTRPT       = CYLBESTD0 + 22,
    CYLBESTBMASSTRIPCHI2   = CYLBESTD0 + 23,
    CYLBESTBMASSTRDOCACHI2 = CYLBESTD0 + 24,

    // Particle Flow related variables
    PFCHARGEDMASS   = CYLBESTBMASSTRDOCACHI2 + 1,
    PFALLMASS       = PFCHARGEDMASS + 1,
    PFCHARGEDTOTPT  = PFCHARGEDMASS + 2,
    PFALLTOTPT      = PFCHARGEDMASS + 3,
    PFCHARGEDMISSPT = PFCHARGEDMASS + 4,
    PFALLMISSPT     = PFCHARGEDMASS + 5,

    BSMUMUTRACKMINUSISOTWO = PFALLMISSPT + 1,
    BSMUMUTRACKPLUSISOTWO  = BSMUMUTRACKMINUSISOTWO + 1,
    ISOTWOBODYMASSISOPLUS  = BSMUMUTRACKMINUSISOTWO + 2,
    ISOTWOBODYMASSISOMINUS = BSMUMUTRACKMINUSISOTWO + 3,
    ISOTWOBODYCHI2ISOPLUS  = BSMUMUTRACKMINUSISOTWO + 4,
    ISOTWOBODYCHI2ISOMINUS = BSMUMUTRACKMINUSISOTWO + 5,
    ISOTWOBODYISO5PLUS     = BSMUMUTRACKMINUSISOTWO + 6,
    ISOTWOBODYISO5MINUS    = BSMUMUTRACKMINUSISOTWO + 7,
    ISOTWOBODYQMINUS       = BSMUMUTRACKMINUSISOTWO + 8,
    ISOTWOBODYQPLUS        = BSMUMUTRACKMINUSISOTWO + 9,

    // Velo phi hits vector size in the transverse plane
    VELOPHIHITSVECTORSIZE = ISOTWOBODYQPLUS + 1,

    B2KSTARMUMUCLASSICISO = VELOPHIHITSVECTORSIZE + 1,

    B2KSTARMUMUBDTISO = VELOPHIHITSVECTORSIZE + 2,

    BSMUMUTRACKID    = B2KSTARMUMUBDTISO + 1,
    BSMUMUTRACKTOPID = BSMUMUTRACKID + 1,
    BSMUMUPARTID     = BSMUMUTRACKID + 2,
    BSMUMUTOPID      = BSMUMUTRACKID + 3,

    BSTAUTAUTAUISOBDTFIRSTVALUETAUP  = BSMUMUTOPID + 1,
    BSTAUTAUTAUISOBDTSECONDVALUETAUP = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 1,
    BSTAUTAUTAUISOBDTTHIRDVALUETAUP  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 2,
    BSTAUTAUTAUISOBDTFIRSTVALUETAUM  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 3,
    BSTAUTAUTAUISOBDTSECONDVALUETAUM = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 4,
    BSTAUTAUTAUISOBDTTHIRDVALUETAUM  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 5,
    BSTAUTAUTAUISOFIRSTVALUETAUP     = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 6,
    BSTAUTAUTAUISOSECONDVALUETAUP    = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 7,
    BSTAUTAUTAUISOFIRSTVALUETAUM     = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 8,
    BSTAUTAUTAUISOSECONDVALUETAUM    = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 9,

    BSTAUTAUMUONISOFIRSTVALUE     = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 10,
    BSTAUTAUMUONISOSECONDVALUE    = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 11,
    BSTAUTAUMUONISOBDTFIRSTVALUE  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 12,
    BSTAUTAUMUONISOBDTSECONDVALUE = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 13,
    BSTAUTAUMUONISOBDTTHIRDVALUE  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 14,

    BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIM   = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 15,
    BSTAUTAUTRKISOBDTSECONDVALUETAUPPIM  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 16,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIM   = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 17,
    BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 18,
    BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP1 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 19,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 20,
    BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 21,
    BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP2 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 22,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 23,

    BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIP   = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 24,
    BSTAUTAUTRKISOBDTSECONDVALUETAUMPIP  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 25,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIP   = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 26,
    BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 27,
    BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM1 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 28,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 29,
    BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 30,
    BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM2 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 31,
    BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 32,

    BSTAUTAUTRKISOFIRSTVALUETAUPPIM  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 33,
    BSTAUTAUTRKISOFIRSTVALUETAUPPIP1 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 34,
    BSTAUTAUTRKISOFIRSTVALUETAUPPIP2 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 35,
    BSTAUTAUTRKISOFIRSTVALUETAUMPIP  = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 36,
    BSTAUTAUTRKISOFIRSTVALUETAUMPIM1 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 37,
    BSTAUTAUTRKISOFIRSTVALUETAUMPIM2 = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 38,

    ZVISOTAUP = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 39,
    ZVISOTAUM = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 40,
    ZVISO     = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 41,

    BSTAUTAUCDFISO = BSTAUTAUTAUISOBDTFIRSTVALUETAUP + 42,

    VTXISODETNOPARTWITHINDCHI2WDW = BSTAUTAUCDFISO + 1,
    VTXISODETNOPARTWITHINCHI2WDW  = BSTAUTAUCDFISO + 2,
    VTXISODETSMALLESTCHI2         = BSTAUTAUCDFISO + 3,
    VTXISODETSMALLESTDELTACHI2    = BSTAUTAUCDFISO + 4,

    BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP  = VTXISODETSMALLESTDELTACHI2 + 1,
    BKSTTAUTAUTAUISOBDTSECONDVALUETAUP = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 1,
    BKSTTAUTAUTAUISOBDTTHIRDVALUETAUP  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 2,
    BKSTTAUTAUTAUISOBDTFIRSTVALUETAUM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 3,
    BKSTTAUTAUTAUISOBDTSECONDVALUETAUM = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 4,
    BKSTTAUTAUTAUISOBDTTHIRDVALUETAUM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 5,
    BKSTTAUTAUTAUISOFIRSTVALUETAUP     = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 6,
    BKSTTAUTAUTAUISOSECONDVALUETAUP    = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 7,
    BKSTTAUTAUTAUISOFIRSTVALUETAUM     = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 8,
    BKSTTAUTAUTAUISOSECONDVALUETAUM    = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 9,

    BKSTTAUTAUMUONISOFIRSTVALUE     = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 10,
    BKSTTAUTAUMUONISOSECONDVALUE    = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 11,
    BKSTTAUTAUMUONISOBDTFIRSTVALUE  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 12,
    BKSTTAUTAUMUONISOBDTSECONDVALUE = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 13,
    BKSTTAUTAUMUONISOBDTTHIRDVALUE  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 14,

    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIM   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 15,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 16,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIM   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 17,
    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 18,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP1 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 19,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 20,
    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 21,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP2 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 22,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 23,

    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIP   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 24,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIP  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 25,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIP   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 26,
    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 27,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM1 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 28,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 29,
    BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 30,
    BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM2 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 31,
    BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 32,

    BKSTTAUTAUTRKISOFIRSTVALUETAUPPIM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 33,
    BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP1 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 34,
    BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP2 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 35,
    BKSTTAUTAUTRKISOFIRSTVALUETAUMPIP  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 36,
    BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM1 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 37,
    BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM2 = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 38,

    BKSTTAUTAUTAUISOBDTFIRSTVALUEKST  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 39,
    BKSTTAUTAUTAUISOBDTSECONDVALUEKST = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 40,
    BKSTTAUTAUTAUISOBDTTHIRDVALUEKST  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 41,

    BKSTTAUTAUZVISOTAUP = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 42,
    BKSTTAUTAUZVISOTAUM = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 43,
    BKSTTAUTAUZVISOKST  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 44,

    BKSTTAUTAUCDFISO = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 45,

    BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTK   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 46,
    BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTPI  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 47,
    BKSTTAUTAUTRKISOBDTSECONDVALUEKSTK  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 48,
    BKSTTAUTAUTRKISOBDTSECONDVALUEKSTPI = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 49,
    BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTK   = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 50,
    BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTPI  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 51,

    BKSTTAUTAUMUONISOBDTFIRSTVALUETAUP  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 52,
    BKSTTAUTAUMUONISOBDTSECONDVALUETAUP = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 53,
    BKSTTAUTAUMUONISOBDTTHIRDVALUETAUP  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 54,

    BKSTTAUTAUMUONISOBDTFIRSTVALUETAUM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 55,
    BKSTTAUTAUMUONISOBDTSECONDVALUETAUM = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 56,
    BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM  = BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP + 57,

    CC_ANGLE    = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 1,
    CC_MULT     = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 2,
    CC_SPT      = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 3,
    CC_VPT      = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 4,
    CC_PX       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 5,
    CC_PY       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 6,
    CC_PZ       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 7,
    CC_PASYM    = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 8,
    CC_PTASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 9,
    CC_PXASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 10,
    CC_PYASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 11,
    CC_PZASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 12,
    CC_DELTAETA = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 13,
    CC_DELTAPHI = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 14,
    CC_IT       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 15,
    CC_MAXPT_Q  = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 16,
    CC_MAXPT_PT = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 17,
    CC_MAXPT_PX = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 18,
    CC_MAXPT_PY = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 19,
    CC_MAXPT_PZ = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 20,
    CC_MAXPT_PE = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 21,
    NC_ANGLE    = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 22,
    NC_MULT     = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 23,
    NC_SPT      = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 24,
    NC_VPT      = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 25,
    NC_PX       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 26,
    NC_PY       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 27,
    NC_PZ       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 28,
    NC_PASYM    = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 29,
    NC_PTASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 30,
    NC_PXASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 31,
    NC_PYASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 32,
    NC_PZASYM   = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 33,
    NC_DELTAETA = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 34,
    NC_DELTAPHI = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 35,
    NC_IT       = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 36,
    NC_MAXPT_Q  = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 37,
    NC_MAXPT_PT = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 38,
    NC_MAXPT_PX = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 39,
    NC_MAXPT_PY = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 40,
    NC_MAXPT_PZ = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 41,
    NC_MAXPT_PE = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 42,
    CCNC_IT     = BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM + 43,

    // muon candidate track isolation in muon detector
    ISOMUM2 = CCNC_IT + 1,
    ISOMUM3 = CCNC_IT + 2,
    ISOMUM4 = CCNC_IT + 3,
    ISOMUM5 = CCNC_IT + 4,

    // muon chi2 with correlation
    MUONCORRCHI2PERDOF = ISOMUM5 + 1,
    MU_BDT             = MUONCORRCHI2PERDOF + 1,

    // velo track match
    VTM_R       = MU_BDT + 1,
    VTM_NTRACKS = MU_BDT + 2,
    VTM_TX      = MU_BDT + 3,
    VTM_TY      = MU_BDT + 4,
    VTM_TPX     = MU_BDT + 5,
    VTM_TPY     = MU_BDT + 6,
    VTM_TPZ     = MU_BDT + 7,
    VTM_IP      = MU_BDT + 8,

    // KstarMuMu isolations
    MU_SLL_ISO_1 = VTM_IP + 1,
    MU_SLL_ISO_2 = VTM_IP + 2,

    // New vertex isolation variables
    NEWVTXISONUMVTX    = MU_SLL_ISO_2 + 1,    // NumVtxWithinDeltaChi2Window
    NEWVTXISOTRKRELD0  = NEWVTXISONUMVTX + 1, // SmallestTrackD0WrtVtx
    NEWVTXISOTRKDCHI2  = NEWVTXISONUMVTX + 2, // SmallestTrackD0Chi2WrtVtx
    NEWVTXISODCHI2MASS = NEWVTXISONUMVTX + 3, // SmallestDeltaChi2Mass

    // Variables for RelInfoGammaIso
    GAMMAISONPI0  = NEWVTXISODCHI2MASS + 1, // Number of pi0
    GAMMAISONETA  = NEWVTXISODCHI2MASS + 2, // Number of Eta
    GAMMAISODMPI0 = NEWVTXISODCHI2MASS + 3, // DMass of the pi0
    GAMMAISODMETA = NEWVTXISODCHI2MASS + 4, // DMass of the Eta

    // New Particle Flow related variables
    PFCHARGEDTOTPX  = GAMMAISODMETA + 1,
    PFALLTOTPX      = PFCHARGEDTOTPX + 1,
    PFCHARGEDMISSPX = PFCHARGEDTOTPX + 2,
    PFALLMISSPX     = PFCHARGEDTOTPX + 3,
    PFCHARGEDTOTPY  = PFCHARGEDTOTPX + 4,
    PFALLTOTPY      = PFCHARGEDTOTPX + 5,
    PFCHARGEDMISSPY = PFCHARGEDTOTPX + 6,
    PFALLMISSPY     = PFCHARGEDTOTPX + 7,

    // Variables for RelInfoTrackIsolationBDT2
    TRKISOBDTLONG        = PFALLMISSPY + 1,
    TRKISOBDTLONG_D1MAX1 = TRKISOBDTLONG + 1,
    TRKISOBDTLONG_D1MAX2 = TRKISOBDTLONG + 2,
    TRKISOBDTLONG_D1MAX3 = TRKISOBDTLONG + 3,
    TRKISOBDTLONG_D2MAX1 = TRKISOBDTLONG + 4,
    TRKISOBDTLONG_D2MAX2 = TRKISOBDTLONG + 5,
    TRKISOBDTLONG_D2MAX3 = TRKISOBDTLONG + 6,
    TRKISOBDTVELO        = TRKISOBDTLONG + 7,
    TRKISOBDTVELO_D1MAX1 = TRKISOBDTLONG + 8,
    TRKISOBDTVELO_D1MAX2 = TRKISOBDTLONG + 9,
    TRKISOBDTVELO_D1MAX3 = TRKISOBDTLONG + 10,
    TRKISOBDTVELO_D2MAX1 = TRKISOBDTLONG + 11,
    TRKISOBDTVELO_D2MAX2 = TRKISOBDTLONG + 12,
    TRKISOBDTVELO_D2MAX3 = TRKISOBDTLONG + 13,
    BS2MUMUBDT           = TRKISOBDTLONG + 14,

    // KstarMuMu HAD isolations
    K_SLL_ISO_HAD  = BS2MUMUBDT + 1,
    PI_SLL_ISO_HAD = BS2MUMUBDT + 2,
    // END must be the highest enum+1
    END = PI_SLL_ISO_HAD + 1

  };

  static std::string nameByIndex( int key ) {
    switch ( key ) {
    case RelatedInfoNamed::CONEANGLE:
      return "CONEANGLE";
    case RelatedInfoNamed::CONEMULT:
      return "CONEMULT";
    case RelatedInfoNamed::CONEPX:
      return "CONEPX";
    case RelatedInfoNamed::CONEPY:
      return "CONEPY";
    case RelatedInfoNamed::CONEPZ:
      return "CONEPZ";
    case RelatedInfoNamed::CONEP:
      return "CONEP";
    case RelatedInfoNamed::CONEPT:
      return "CONEPT";
    case RelatedInfoNamed::CONEPXASYM:
      return "CONEPXASYM";
    case RelatedInfoNamed::CONEPYASYM:
      return "CONEPYASYM";
    case RelatedInfoNamed::CONEPZASYM:
      return "CONEPZASYM";
    case RelatedInfoNamed::CONEPASYM:
      return "CONEPASYM";
    case RelatedInfoNamed::CONEPTASYM:
      return "CONEPTASYM";
    case RelatedInfoNamed::CONEDELTAETA:
      return "CONEDELTAETA";
    case RelatedInfoNamed::CONEDELTAPHI:
      return "CONEDELTAPHI";

    case RelatedInfoNamed::EWCONEANGLE:
      return "EWCONEANGLE";
    case RelatedInfoNamed::EWCONEMULT:
      return "EWCONEMULT";
    case RelatedInfoNamed::EWCONEPX:
      return "EWCONEPX";
    case RelatedInfoNamed::EWCONEPY:
      return "EWCONEPY";
    case RelatedInfoNamed::EWCONEPZ:
      return "EWCONEPZ";
    case RelatedInfoNamed::EWCONEVP:
      return "EWCONEVP";
    case RelatedInfoNamed::EWCONEVPT:
      return "EWCONEVPT";
    case RelatedInfoNamed::EWCONESP:
      return "EWCONESP";
    case RelatedInfoNamed::EWCONESPT:
      return "EWCONESPT";
    case RelatedInfoNamed::EWCONETP:
      return "EWCONETP";
    case RelatedInfoNamed::EWCONETPT:
      return "EWCONETPT";
    case RelatedInfoNamed::EWCONEMINPTE:
      return "EWCONEMINPTE";
    case RelatedInfoNamed::EWCONEMAXPTE:
      return "EWCONEMAXPTE";
    case RelatedInfoNamed::EWCONEMINPTMU:
      return "EWCONEMINPTMU";
    case RelatedInfoNamed::EWCONEMAXPTMU:
      return "EWCONEMAXPTMU";
    case RelatedInfoNamed::EWCONENMULT:
      return "EWCONENMULT";
    case RelatedInfoNamed::EWCONENPX:
      return "EWCONENPX";
    case RelatedInfoNamed::EWCONENPY:
      return "EWCONENPY";
    case RelatedInfoNamed::EWCONENPZ:
      return "EWCONENPZ";
    case RelatedInfoNamed::EWCONENVP:
      return "EWCONENVP";
    case RelatedInfoNamed::EWCONENVPT:
      return "EWCONENVPT";
    case RelatedInfoNamed::EWCONENSP:
      return "EWCONENSP";
    case RelatedInfoNamed::EWCONENSPT:
      return "EWCONENSPT";

    case RelatedInfoNamed::VTXISONUMVTX:
      return "VTXISONUMVTX";
    case RelatedInfoNamed::VTXISODCHI2ONETRACK:
      return "VTXISODCHI2ONETRACK";
    case RelatedInfoNamed::VTXISODCHI2MASSONETRACK:
      return "VTXISODCHI2MASSONETRACK";
    case RelatedInfoNamed::VTXISODCHI2TWOTRACK:
      return "VTXISODCHI2TWOTRACK";
    case RelatedInfoNamed::VTXISODCHI2MASSTWOTRACK:
      return "VTXISODCHI2MASSTWOTRACK";

    case RelatedInfoNamed::VTXISOBDTHARDFIRSTVALUE:
      return "VTXISOBDTHARDFIRSTVALUE";
    case RelatedInfoNamed::VTXISOBDTHARDSECONDVALUE:
      return "VTXISOBDTHARDSECONDVALUE";
    case RelatedInfoNamed::VTXISOBDTHARDTHIRDVALUE:
      return "VTXISOBDTHARDTHIRDVALUE";
    case RelatedInfoNamed::VTXISOBDTSOFTFIRSTVALUE:
      return "VTXISOBDTSOFTFIRSTVALUE";
    case RelatedInfoNamed::VTXISOBDTSOFTSECONDVALUE:
      return "VTXISOBDTSOFTSECONDVALUE";
    case RelatedInfoNamed::VTXISOBDTSOFTTHIRDVALUE:
      return "VTXISOBDTSOFTTHIRDVALUE";
    case RelatedInfoNamed::TRKISOBDTFIRSTVALUE:
      return "TRKISOBDTFIRSTVALUE";
    case RelatedInfoNamed::TRKISOBDTSECONDVALUE:
      return "TRKISOBDTSECONDVALUE";
    case RelatedInfoNamed::TRKISOBDTTHIRDVALUE:
      return "TRKISOBDTTHIRDVALUE";

    case RelatedInfoNamed::TRKISOBDTLONG:
      return "TRKISOBDTLONG";
    case RelatedInfoNamed::TRKISOBDTLONG_D1MAX1:
      return "TRKISOBDTLONG_D1MAX1";
    case RelatedInfoNamed::TRKISOBDTLONG_D1MAX2:
      return "TRKISOBDTLONG_D1MAX2";
    case RelatedInfoNamed::TRKISOBDTLONG_D1MAX3:
      return "TRKISOBDTLONG_D1MAX3";
    case RelatedInfoNamed::TRKISOBDTLONG_D2MAX1:
      return "TRKISOBDTLONG_D2MAX1";
    case RelatedInfoNamed::TRKISOBDTLONG_D2MAX2:
      return "TRKISOBDTLONG_D2MAX2";
    case RelatedInfoNamed::TRKISOBDTLONG_D2MAX3:
      return "TRKISOBDTLONG_D2MAX3";
    case RelatedInfoNamed::TRKISOBDTVELO:
      return "TRKISOBDTVELO";
    case RelatedInfoNamed::TRKISOBDTVELO_D1MAX1:
      return "TRKISOBDTVELO_D1MAX1";
    case RelatedInfoNamed::TRKISOBDTVELO_D1MAX2:
      return "TRKISOBDTVELO_D1MAX2";
    case RelatedInfoNamed::TRKISOBDTVELO_D1MAX3:
      return "TRKISOBDTVELO_D1MAX3";
    case RelatedInfoNamed::TRKISOBDTVELO_D2MAX1:
      return "TRKISOBDTVELO_D2MAX1";
    case RelatedInfoNamed::TRKISOBDTVELO_D2MAX2:
      return "TRKISOBDTVELO_D2MAX2";
    case RelatedInfoNamed::TRKISOBDTVELO_D2MAX3:
      return "TRKISOBDTVELO_D2MAX3";
    case RelatedInfoNamed::BS2MUMUBDT:
      return "BS2MUMUBDT";

    case RelatedInfoNamed::BSMUMUCDFISO:
      return "BSMUMUCDFISO";
    case RelatedInfoNamed::BSMUMUOTHERBMAG:
      return "BSMUMUOTHERBMAG";
    case RelatedInfoNamed::BSMUMUOTHERBANGLE:
      return "BSMUMUOTHERBANGLE";
    case RelatedInfoNamed::BSMUMUOTHERBBOOSTMAG:
      return "BSMUMUOTHERBBOOSTMAG";
    case RelatedInfoNamed::BSMUMUOTHERBBOOSTANGLE:
      return "BSMUMUOTHERBBOOSTANGLE";
    case RelatedInfoNamed::BSMUMUTRACKPLUSISO:
      return "BSMUMUTRACKPLUSISO";
    case RelatedInfoNamed::BSMUMUTRACKMINUSISO:
      return "BSMUMUTRACKMINUSISO";
    case RelatedInfoNamed::BSMUMUOTHERBTRACKS:
      return "BSMUMUOTHERBTRACKS";

    case RelatedInfoNamed::JETNOMU1PX:
      return "JETNOMU1PX";
    case RelatedInfoNamed::JETNOMU1PY:
      return "JETNOMU1PY";
    case RelatedInfoNamed::JETNOMU1PZ:
      return "JETNOMU1PZ";
    case RelatedInfoNamed::JETNOMU1PT:
      return "JETNOMU1PT";
    case RelatedInfoNamed::JETNOMU1JETWIDTH:
      return "JETNOMU1JETWIDTH";
    case RelatedInfoNamed::JETNOMU1NNTAG:
      return "JETNOMU1NNTAG";
    case RelatedInfoNamed::JETNOMU1MNF:
      return "JETNOMU1MNF";
    case RelatedInfoNamed::JETNOMU2PX:
      return "JETNOMU2PX";
    case RelatedInfoNamed::JETNOMU2PY:
      return "JETNOMU2PY";
    case RelatedInfoNamed::JETNOMU2PZ:
      return "JETNOMU2PZ";
    case RelatedInfoNamed::JETNOMU2PT:
      return "JETNOMU2PT";
    case RelatedInfoNamed::JETNOMU2JETWIDTH:
      return "JETNOMU2JETWIDTH";
    case RelatedInfoNamed::JETNOMU2NNTAG:
      return "JETNOMU2NNTAG";
    case RelatedInfoNamed::JETNOMU2MNF:
      return "JETNOMU2MNF";
    case RelatedInfoNamed::JETNOMU3PX:
      return "JETNOMU3PX";
    case RelatedInfoNamed::JETNOMU3PY:
      return "JETNOMU3PY";
    case RelatedInfoNamed::JETNOMU3PZ:
      return "JETNOMU3PZ";
    case RelatedInfoNamed::JETNOMU3PT:
      return "JETNOMU3PT";
    case RelatedInfoNamed::JETNOMU3JETWIDTH:
      return "JETNOMU3JETWIDTH";
    case RelatedInfoNamed::JETNOMU3NNTAG:
      return "JETNOMU3NNTAG";
    case RelatedInfoNamed::JETNOMU3MNF:
      return "JETNOMU3MNF";
    case RelatedInfoNamed::JETMU1PX:
      return "JETMU1PX";
    case RelatedInfoNamed::JETMU1PY:
      return "JETMU1PY";
    case RelatedInfoNamed::JETMU1PZ:
      return "JETMU1PZ";
    case RelatedInfoNamed::JETMU1PT:
      return "JETMU1PT";
    case RelatedInfoNamed::JETMU1JETWIDTH:
      return "JETMU1JETWIDTH";
    case RelatedInfoNamed::JETMU1NNTAG:
      return "JETMU1NNTAG";
    case RelatedInfoNamed::JETMU1MNF:
      return "JETMU1MNF";
    case RelatedInfoNamed::JETMU2PX:
      return "JETMU2PX";
    case RelatedInfoNamed::JETMU2PY:
      return "JETMU2PY";
    case RelatedInfoNamed::JETMU2PZ:
      return "JETMU2PZ";
    case RelatedInfoNamed::JETMU2PT:
      return "JETMU2PT";
    case RelatedInfoNamed::JETMU2JETWIDTH:
      return "JETMU2JETWIDTH";
    case RelatedInfoNamed::JETMU2NNTAG:
      return "JETMU2NNTAG";
    case RelatedInfoNamed::JETMU2MNF:
      return "JETMU2MNF";
    case RelatedInfoNamed::JETBPX:
      return "JETBPX";
    case RelatedInfoNamed::JETBPY:
      return "JETBPY";
    case RelatedInfoNamed::JETBPZ:
      return "JETBPZ";
    case RelatedInfoNamed::JETBPT:
      return "JETBPT";
    case RelatedInfoNamed::JETBJETWIDTH:
      return "JETBJETWIDTH";
    case RelatedInfoNamed::JETBNNTAG:
      return "JETBNNTAG";
    case RelatedInfoNamed::JETBMNF:
      return "JETBMNF";

    case RelatedInfoNamed::CYLBESTD0:
      return "CYLBESTD0";
    case RelatedInfoNamed::CYLBESTKS:
      return "CYLBESTKS";
    case RelatedInfoNamed::CYLTRACKSNCYL:
      return "CYLTRACKSNCYL";
    case RelatedInfoNamed::CYLMINPT:
      return "CYLMINPT";
    case RelatedInfoNamed::CYLMAXPT:
      return "CYLMAXPT";
    case RelatedInfoNamed::CYLAVPT:
      return "CYLAVPT";
    case RelatedInfoNamed::CYLMAXPTTRIPCHI2:
      return "CYLMAXPTTRIPCHI2";
    case RelatedInfoNamed::CYLMAXPTTRDOCACHI2:
      return "CYLMAXPTTRDOCACHI2";
    case RelatedInfoNamed::CYLMAXPTTRBMASS:
      return "CYLMAXPTTRBMASS";
    case RelatedInfoNamed::CYLMINIPCHI2:
      return "CYLMINIPCHI2";
    case RelatedInfoNamed::CYLMAXIPCHI2:
      return "CYLMAXIPCHI2";
    case RelatedInfoNamed::CYLAVIPCHI2:
      return "CYLAVIPCHI2";
    case RelatedInfoNamed::CYLMAXIPCHI2TRPT:
      return "CYLMAXIPCHI2TRPT";
    case RelatedInfoNamed::CYLMAXIPCHI2TRDOCACHI2:
      return "CYLMAXIPCHI2TRDOCACHI2";
    case RelatedInfoNamed::CYLMAXIPCHI2TRBMASS:
      return "CYLMAXIPCHI2TRBMASS";
    case RelatedInfoNamed::CYLMINDOCACHI2:
      return "CYLMINDOCACHI2";
    case RelatedInfoNamed::CYLMAXDOCACHI2:
      return "CYLMAXDOCACHI2";
    case RelatedInfoNamed::CYLAVDOCACHI2:
      return "CYLAVDOCACHI2";
    case RelatedInfoNamed::CYLMINDOCATRPT:
      return "CYLMINDOCATRPT";
    case RelatedInfoNamed::CYLMINDOCATRIPCHI2:
      return "CYLMINDOCATRIPCHI2";
    case RelatedInfoNamed::CYLMINDOCATRBMASS:
      return "CYLMINDOCATRBMASS";
    case RelatedInfoNamed::CYLBESTBMASS:
      return "CYLBESTBMASS";
    case RelatedInfoNamed::CYLBESTBMASSTRPT:
      return "CYLBESTBMASSTRPT";
    case RelatedInfoNamed::CYLBESTBMASSTRIPCHI2:
      return "CYLBESTBMASSTRIPCHI2";
    case RelatedInfoNamed::CYLBESTBMASSTRDOCACHI2:
      return "CYLBESTBMASSTRDOCACHI2";
    case RelatedInfoNamed::PFCHARGEDMASS:
      return "PFCHARGEDMASS";
    case RelatedInfoNamed::PFALLMASS:
      return "PFALLMASS";
    case RelatedInfoNamed::PFCHARGEDTOTPT:
      return "PFCHARGEDTOTPT";
    case RelatedInfoNamed::PFALLTOTPT:
      return "PFALLTOTPT";
    case RelatedInfoNamed::PFCHARGEDMISSPT:
      return "PFCHARGEDMISSPT";
    case RelatedInfoNamed::PFALLMISSPT:
      return "PFALLMISSPT";
    case RelatedInfoNamed::PFCHARGEDTOTPX:
      return "PFCHARGEDTOTPX";
    case RelatedInfoNamed::PFALLTOTPX:
      return "PFALLTOTPX";
    case RelatedInfoNamed::PFCHARGEDMISSPX:
      return "PFCHARGEDMISSPX";
    case RelatedInfoNamed::PFALLMISSPX:
      return "PFALLMISSPX";
    case RelatedInfoNamed::PFCHARGEDTOTPY:
      return "PFCHARGEDTOTPY";
    case RelatedInfoNamed::PFALLTOTPY:
      return "PFALLTOTPY";
    case RelatedInfoNamed::PFCHARGEDMISSPY:
      return "PFCHARGEDMISSPY";
    case RelatedInfoNamed::PFALLMISSPY:
      return "PFALLMISSPY";

    case RelatedInfoNamed::BSMUMUTRACKMINUSISOTWO:
      return "BSMUMUTRACKMINUSISOTWO";
    case RelatedInfoNamed::BSMUMUTRACKPLUSISOTWO:
      return "BSMUMUTRACKPLUSISOTWO";
    case RelatedInfoNamed::ISOTWOBODYMASSISOPLUS:
      return "ISOTWOBODYMASSISOPLUS";
    case RelatedInfoNamed::ISOTWOBODYMASSISOMINUS:
      return "ISOTWOBODYMASSISOMINUS";
    case RelatedInfoNamed::ISOTWOBODYCHI2ISOPLUS:
      return "ISOTWOBODYCHI2ISOPLUS";
    case RelatedInfoNamed::ISOTWOBODYCHI2ISOMINUS:
      return "ISOTWOBODYCHI2ISOMINUS";
    case RelatedInfoNamed::ISOTWOBODYISO5PLUS:
      return "ISOTWOBODYISO5PLUS";
    case RelatedInfoNamed::ISOTWOBODYISO5MINUS:
      return "ISOTWOBODYISO5MINUS";
    case RelatedInfoNamed::ISOTWOBODYQMINUS:
      return "ISOTWOBODYQMINUS";
    case RelatedInfoNamed::ISOTWOBODYQPLUS:
      return "ISOTWOBODYQPLUS";

    case RelatedInfoNamed::VELOPHIHITSVECTORSIZE:
      return "VELOPHIHITSVECTORSIZE";
    case RelatedInfoNamed::B2KSTARMUMUCLASSICISO:
      return "B2KSTARMUMUCLASSICISO";

    case RelatedInfoNamed::B2KSTARMUMUBDTISO:
      return "B2KSTARMUMUBDTISO";

    case RelatedInfoNamed::BSMUMUTRACKID:
      return "BSMUMUTRACKID";
    case RelatedInfoNamed::BSMUMUTRACKTOPID:
      return "BSMUMUTRACKTOPID";
    case RelatedInfoNamed::BSMUMUPARTID:
      return "BSMUMUPARTID";
    case RelatedInfoNamed::BSMUMUTOPID:
      return "BSMUMUTOPID";

    case RelatedInfoNamed::BSTAUTAUTAUISOBDTFIRSTVALUETAUP:
      return "BSTAUTAUTAUISOBDTFIRSTVALUETAUP";
    case RelatedInfoNamed::BSTAUTAUTAUISOBDTSECONDVALUETAUP:
      return "BSTAUTAUTAUISOBDTSECONDVALUETAUP";
    case RelatedInfoNamed::BSTAUTAUTAUISOBDTTHIRDVALUETAUP:
      return "BSTAUTAUTAUISOBDTTHIRDVALUETAUP";
    case RelatedInfoNamed::BSTAUTAUTAUISOBDTFIRSTVALUETAUM:
      return "BSTAUTAUTAUISOBDTFIRSTVALUETAUM";
    case RelatedInfoNamed::BSTAUTAUTAUISOBDTSECONDVALUETAUM:
      return "BSTAUTAUTAUISOBDTSECONDVALUETAUM";
    case RelatedInfoNamed::BSTAUTAUTAUISOBDTTHIRDVALUETAUM:
      return "BSTAUTAUTAUISOBDTTHIRDVALUETAUM";
    case RelatedInfoNamed::BSTAUTAUTAUISOFIRSTVALUETAUP:
      return "BSTAUTAUTAUISOFIRSTVALUETAUP";
    case RelatedInfoNamed::BSTAUTAUTAUISOSECONDVALUETAUP:
      return "BSTAUTAUTAUISOSECONDVALUETAUP";
    case RelatedInfoNamed::BSTAUTAUTAUISOFIRSTVALUETAUM:
      return "BSTAUTAUTAUISOFIRSTVALUETAUM";
    case RelatedInfoNamed::BSTAUTAUTAUISOSECONDVALUETAUM:
      return "BSTAUTAUTAUISOSECONDVALUETAUM";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIM:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIM";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUPPIM:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUPPIM";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIM:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIM";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP1:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP2:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUPPIP2";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIP:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIP";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUMPIP:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUMPIP";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIP:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIP";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM1:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2:
      return "BSTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM2:
      return "BSTAUTAUTRKISOBDTSECONDVALUETAUMPIM2";
    case RelatedInfoNamed::BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2:
      return "BSTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUPPIM:
      return "BSTAUTAUTRKISOFIRSTVALUETAUPPIM";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUPPIP1:
      return "BSTAUTAUTRKISOFIRSTVALUETAUPPIP1";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUPPIP2:
      return "BSTAUTAUTRKISOFIRSTVALUETAUPPIP2";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUMPIP:
      return "BSTAUTAUTRKISOFIRSTVALUETAUMPIP";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUMPIM1:
      return "BSTAUTAUTRKISOFIRSTVALUETAUMPIM1";
    case RelatedInfoNamed::BSTAUTAUTRKISOFIRSTVALUETAUMPIM2:
      return "BSTAUTAUTRKISOFIRSTVALUETAUMPIM2";
    case RelatedInfoNamed::BSTAUTAUMUONISOBDTFIRSTVALUE:
      return "BSTAUTAUMUONISOBDTFIRSTVALUE";
    case RelatedInfoNamed::BSTAUTAUMUONISOBDTSECONDVALUE:
      return "BSTAUTAUMUONISOBDTSECONDVALUE";
    case RelatedInfoNamed::BSTAUTAUMUONISOBDTTHIRDVALUE:
      return "BSTAUTAUMUONISOBDTTHIRDVALUE";
    case RelatedInfoNamed::BSTAUTAUMUONISOFIRSTVALUE:
      return "BSTAUTAUMUONISOFIRSTVALUE";
    case RelatedInfoNamed::BSTAUTAUMUONISOSECONDVALUE:
      return "BSTAUTAUMUONISOSECONDVALUE";
    case RelatedInfoNamed::BSTAUTAUCDFISO:
      return "BSTAUTAUCDFISO";
    case RelatedInfoNamed::ZVISO:
      return "ZVISO";
    case RelatedInfoNamed::ZVISOTAUP:
      return "ZVISOTAUP";
    case RelatedInfoNamed::ZVISOTAUM:
      return "ZVISOTAUM";

    case RelatedInfoNamed::VTXISODETNOPARTWITHINDCHI2WDW:
      return "VTXISODETNOPARTWITHINDCHI2WDW";
    case RelatedInfoNamed::VTXISODETNOPARTWITHINCHI2WDW:
      return "VTXISODETNOPARTWITHINCHI2WDW";
    case RelatedInfoNamed::VTXISODETSMALLESTCHI2:
      return "VTXISODETSMALLESTCHI2";
    case RelatedInfoNamed::VTXISODETSMALLESTDELTACHI2:
      return "VTXISODETSMALLESTDELTACHI2";

    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP:
      return "BKSTTAUTAUTAUISOBDTFIRSTVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTSECONDVALUETAUP:
      return "BKSTTAUTAUTAUISOBDTSECONDVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTTHIRDVALUETAUP:
      return "BKSTTAUTAUTAUISOBDTTHIRDVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTFIRSTVALUETAUM:
      return "BKSTTAUTAUTAUISOBDTFIRSTVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTSECONDVALUETAUM:
      return "BKSTTAUTAUTAUISOBDTSECONDVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTTHIRDVALUETAUM:
      return "BKSTTAUTAUTAUISOBDTTHIRDVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOFIRSTVALUETAUP:
      return "BKSTTAUTAUTAUISOFIRSTVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOSECONDVALUETAUP:
      return "BKSTTAUTAUTAUISOSECONDVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOFIRSTVALUETAUM:
      return "BKSTTAUTAUTAUISOFIRSTVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOSECONDVALUETAUM:
      return "BKSTTAUTAUTAUISOSECONDVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIM:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIM";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIM:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIM";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIM:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIM";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP1:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUPPIP2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP2:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUPPIP2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUPPIP2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIP:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIP";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIP:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIP";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIP:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIP";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM1:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUETAUMPIM2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM2:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUETAUMPIM2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUETAUMPIM2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUPPIM:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUPPIM";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP1:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP2:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUPPIP2";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUMPIP:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUMPIP";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM1:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM1";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM2:
      return "BKSTTAUTAUTRKISOFIRSTVALUETAUMPIM2";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTFIRSTVALUE:
      return "BKSTTAUTAUMUONISOBDTFIRSTVALUE";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTSECONDVALUE:
      return "BKSTTAUTAUMUONISOBDTSECONDVALUE";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTTHIRDVALUE:
      return "BKSTTAUTAUMUONISOBDTTHIRDVALUE";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOFIRSTVALUE:
      return "BKSTTAUTAUMUONISOFIRSTVALUE";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOSECONDVALUE:
      return "BKSTTAUTAUMUONISOSECONDVALUE";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTFIRSTVALUEKST:
      return "BKSTTAUTAUTAUISOBDTFIRSTVALUEKST";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTSECONDVALUEKST:
      return "BKSTTAUTAUTAUISOBDTSECONDVALUEKST";
    case RelatedInfoNamed::BKSTTAUTAUTAUISOBDTTHIRDVALUEKST:
      return "BKSTTAUTAUTAUISOBDTTHIRDVALUEKST";

    case RelatedInfoNamed::BKSTTAUTAUZVISOTAUP:
      return "BKSTTAUTAUZVISOTAUP";
    case RelatedInfoNamed::BKSTTAUTAUZVISOTAUM:
      return "BKSTTAUTAUZVISOTAUM";
    case RelatedInfoNamed::BKSTTAUTAUZVISOKST:
      return "BKSTTAUTAUZVISOKST";
    case RelatedInfoNamed::BKSTTAUTAUCDFISO:
      return "BKSTTAUTAUCDFISO";

    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTK:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTK";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTPI:
      return "BKSTTAUTAUTRKISOBDTFIRSTVALUEKSTPI";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUEKSTK:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUEKSTK";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTSECONDVALUEKSTPI:
      return "BKSTTAUTAUTRKISOBDTSECONDVALUEKSTPI";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTK:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTK";
    case RelatedInfoNamed::BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTPI:
      return "BKSTTAUTAUTRKISOBDTTHIRDVALUEKSTPI";

    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTFIRSTVALUETAUP:
      return "BKSTTAUTAUMUONISOBDTFIRSTVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTSECONDVALUETAUP:
      return "BKSTTAUTAUMUONISOBDTSECONDVALUETAUP";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTTHIRDVALUETAUP:
      return "BKSTTAUTAUMUONISOBDTTHIRDVALUETAUP";

    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTFIRSTVALUETAUM:
      return "BKSTTAUTAUMUONISOBDTFIRSTVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTSECONDVALUETAUM:
      return "BKSTTAUTAUMUONISOBDTSECONDVALUETAUM";
    case RelatedInfoNamed::BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM:
      return "BKSTTAUTAUMUONISOBDTTHIRDVALUETAUM";

    case RelatedInfoNamed::CC_ANGLE:
      return "CC_ANGLE";
    case RelatedInfoNamed::CC_MULT:
      return "CC_MULT";
    case RelatedInfoNamed::CC_SPT:
      return "CC_SPT";
    case RelatedInfoNamed::CC_VPT:
      return "CC_VPT";
    case RelatedInfoNamed::CC_PX:
      return "CC_PX";
    case RelatedInfoNamed::CC_PY:
      return "CC_PY";
    case RelatedInfoNamed::CC_PZ:
      return "CC_PZ";
    case RelatedInfoNamed::CC_PASYM:
      return "CC_PASYM";
    case RelatedInfoNamed::CC_PTASYM:
      return "CC_PTASYM";
    case RelatedInfoNamed::CC_PXASYM:
      return "CC_PXASYM";
    case RelatedInfoNamed::CC_PYASYM:
      return "CC_PYASYM";
    case RelatedInfoNamed::CC_PZASYM:
      return "CC_PZASYM";
    case RelatedInfoNamed::CC_DELTAETA:
      return "CC_DELTAETA";
    case RelatedInfoNamed::CC_DELTAPHI:
      return "CC_DELTAPHI";
    case RelatedInfoNamed::CC_IT:
      return "CC_IT";
    case RelatedInfoNamed::CC_MAXPT_Q:
      return "CC_MAXPT_Q";
    case RelatedInfoNamed::CC_MAXPT_PT:
      return "CC_MAXPT_PT";
    case RelatedInfoNamed::CC_MAXPT_PX:
      return "CC_MAXPT_PX";
    case RelatedInfoNamed::CC_MAXPT_PY:
      return "CC_MAXPT_PY";
    case RelatedInfoNamed::CC_MAXPT_PZ:
      return "CC_MAXPT_PZ";
    case RelatedInfoNamed::CC_MAXPT_PE:
      return "CC_MAXPT_PE";
    case RelatedInfoNamed::NC_ANGLE:
      return "NC_ANGLE";
    case RelatedInfoNamed::NC_MULT:
      return "NC_MULT";
    case RelatedInfoNamed::NC_SPT:
      return "NC_SPT";
    case RelatedInfoNamed::NC_VPT:
      return "NC_VPT";
    case RelatedInfoNamed::NC_PX:
      return "NC_PX";
    case RelatedInfoNamed::NC_PY:
      return "NC_PY";
    case RelatedInfoNamed::NC_PZ:
      return "NC_PZ";
    case RelatedInfoNamed::NC_PASYM:
      return "NC_PASYM";
    case RelatedInfoNamed::NC_PTASYM:
      return "NC_PTASYM";
    case RelatedInfoNamed::NC_PXASYM:
      return "NC_PXASYM";
    case RelatedInfoNamed::NC_PYASYM:
      return "NC_PYASYM";
    case RelatedInfoNamed::NC_PZASYM:
      return "NC_PZASYM";
    case RelatedInfoNamed::NC_DELTAETA:
      return "NC_DELTAETA";
    case RelatedInfoNamed::NC_DELTAPHI:
      return "NC_DELTAPHI";
    case RelatedInfoNamed::NC_IT:
      return "NC_IT";
    case RelatedInfoNamed::NC_MAXPT_Q:
      return "NC_MAXPT_Q";
    case RelatedInfoNamed::NC_MAXPT_PT:
      return "NC_MAXPT_PT";
    case RelatedInfoNamed::NC_MAXPT_PX:
      return "NC_MAXPT_PX";
    case RelatedInfoNamed::NC_MAXPT_PY:
      return "NC_MAXPT_PY";
    case RelatedInfoNamed::NC_MAXPT_PZ:
      return "NC_MAXPT_PZ";
    case RelatedInfoNamed::NC_MAXPT_PE:
      return "NC_MAXPT_PE";
    case RelatedInfoNamed::CCNC_IT:
      return "CCNC_IT";

    case RelatedInfoNamed::ISOMUM2:
      return "ISOMUM2";
    case RelatedInfoNamed::ISOMUM3:
      return "ISOMUM3";
    case RelatedInfoNamed::ISOMUM4:
      return "ISOMUM4";
    case RelatedInfoNamed::ISOMUM5:
      return "ISOMUM5";

    case RelatedInfoNamed::MUONCORRCHI2PERDOF:
      return "MUONCORRCHI2PERDOF";
    case RelatedInfoNamed::MU_BDT:
      return "MU_BDT";

    case RelatedInfoNamed::VTM_R:
      return "VTM_R";
    case RelatedInfoNamed::VTM_NTRACKS:
      return "VTM_NTRACKS";
    case RelatedInfoNamed::VTM_TX:
      return "VTM_TX";
    case RelatedInfoNamed::VTM_TY:
      return "VTM_TY";
    case RelatedInfoNamed::VTM_TPX:
      return "VTM_TPX";
    case RelatedInfoNamed::VTM_TPY:
      return "VTM_TPY";
    case RelatedInfoNamed::VTM_TPZ:
      return "VTM_TPZ";
    case RelatedInfoNamed::VTM_IP:
      return "VTM_IP";

    case RelatedInfoNamed::MU_SLL_ISO_1:
      return "MU_SLL_ISO_1";
    case RelatedInfoNamed::MU_SLL_ISO_2:
      return "MU_SLL_ISO_2";
    case RelatedInfoNamed::K_SLL_ISO_HAD:
      return "K_SLL_ISO_HAD";
    case RelatedInfoNamed::PI_SLL_ISO_HAD:
      return "PI_SLL_ISO_HAD";

    case RelatedInfoNamed::NEWVTXISONUMVTX:
      return "NEWVTXISONUMVTX";
    case RelatedInfoNamed::NEWVTXISOTRKRELD0:
      return "NEWVTXISOTRKRELD0";
    case RelatedInfoNamed::NEWVTXISOTRKDCHI2:
      return "NEWVTXISOTRKDCHI2";
    case RelatedInfoNamed::NEWVTXISODCHI2MASS:
      return "NEWVTXISODCHI2MASS";

    case RelatedInfoNamed::GAMMAISONPI0:
      return "GAMMAISONPI0";
    case RelatedInfoNamed::GAMMAISONETA:
      return "GAMMAISONETA";
    case RelatedInfoNamed::GAMMAISODMPI0:
      return "GAMMAISODMPI0";
    case RelatedInfoNamed::GAMMAISODMETA:
      return "GAMMAISODMETA";

    default:
      return "UNKNOWN";
    }
  }

  static short int indexByName( std::string name ) {
    short i;
    for ( i = 0; i < RelatedInfoNamed::END; i++ ) {

      if ( name.compare( nameByIndex( i ) ) == 0 ) return i;
    }
    return RelatedInfoNamed::UNKNOWN;
  }
};

#endif
