/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
#ifndef KALMANFILTER_ERRORCODES_H
#  define KALMANFILTER_ERRORCODES_H 1
// ============================================================================
// Include files
// ============================================================================
/** @file
 *
 *  This file is a part of
 *  <a href="http://cern.ch/lhcb-comp/Analysis/LoKi/index.html">LoKi project:</a>
 *  ``C++ ToolKit for Smart and Friendly Physics Analysis''
 *
 *  This file is a part of LoKi project -
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
 *  contributions and advices from G.Raven, J.van Tilburg,
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 *  @date   2006-04-12
 */
// ============================================================================
namespace LoKi {
  // ==========================================================================
  namespace KalmanFilter {
    // ========================================================================
    /// error codes from various routines
    enum ErrorCodes {
      // ======================================================================
      /// Error in Matrix Inversion (I)
      ErrorInMatrixInversion1 = 403, // Error in Matrix Inversion
      /// Error in Matrix Inversion (II)
      ErrorInMatrixInversion2 = 404, // Error in Matrix Inversion
      /// Error in Matrix Inversion (III)
      ErrorInMatrixInversion3 = 405, // Error in Matrix Inversion
      /// Error in Matrix Inversion (IV)
      ErrorInMatrixInversion4 = 406, // Error in Matrix Inversion
      /// Error in InputData
      ErrorInInputData = 407, // Error in Input  Data
      /// Error from CaloMomentum
      ErrorFromCaloMomentum = 408, // Error from CaloMomentum
      /// Error from Particle Transporter
      ErrorFromParticleTransporter = 410, // Error from particle transporter
      /// Error in "Gamma-Like" particle
      ErrorGammaLikeParticle = 411, // Error in Gamma-Like particle
      /// Error in "DiGamma-Like" particle
      ErrorDiGammaLikeParticle = 412, // Error in DiGamma-Like particle
      /// Error in Gamma-'transport'
      ErrorGammaTransport = 413, // Error in Gamma-transport
      /// Error in Merged-pi0 update
      ErrorMergedPi0Update = 414 // Error in Merged-pi0-update
      // ======================================================================
    };
    // ========================================================================
  } // namespace KalmanFilter
  // ==========================================================================
} //                                                      end of namespace LoKi
// ============================================================================
// The END
// ============================================================================
#endif // KALMANFILTER_ERRORCODES_H
// ============================================================================
