/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// Include files
// ============================================================================
// local
// ============================================================================
#include "VertexFitter.h"
// ============================================================================
/** @file
 *
 *  This file is a part of
 *  <a href="http://cern.ch/lhcb-comp/Analysis/LoKi/index.html">LoKi project:</a>
 *  ``C++ ToolKit for Smart and Friendly Physics Analysis''
 *
 *  This file is a part of LoKi project -
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
 *  contributions and advices from G.Raven, J.van Tilburg,
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya BELYAEV Ivan.Belyaev@nikhef.nl
 *  @date 2010-07-11
 */
// ============================================================================
namespace LoKi {
  // ==========================================================================
  /** @class FastVertexFitter
   *  Preconfigured instance of LoKi::VertexFitetr with smaller number
   *  of iterations and loose tolerances
   *
   *  It used 'loose' convergency srettria:
   *    - delta-distance as 3.0 micrometers
   *    - delta-chi2     as 0.1
   *    - maximal number of iteration for 'fit'    is 5
   *    - maximal number of iteration for 'add'    is 3
   *    - maximal number of iteration for 'remove' is 3
   *    - the special branch for two-body decays is activated
   *
   *  @author Vanya BELYAEV Ivan.Belyaev@nikhef.nl
   *  @date 2010-07-11
   *  @see LoKi::VertexFitter
   */
  class FastVertexFitter : public LoKi::VertexFitter {
  public:
    //=========================================================================
    /** strandard constructor
     *  @see LoKi::VertexFitter
     *  @see GaudiTool
     *  @see AlgTool
     */
    FastVertexFitter( const std::string& type, const std::string& name, const IInterface* parent );
    // ========================================================================
  };
  // ==========================================================================
} //                                                      end of namespace LoKi
// ============================================================================
/*  standard constructor
 *  @see LoKi::VertexFitter
 *  @see GaudiTool
 *  @see AlgTool
 */
// ============================================================================
LoKi::FastVertexFitter::FastVertexFitter( const std::string& type, const std::string& name, const IInterface* parent )
    : LoKi::VertexFitter( type, name, parent ) {
  StatusCode sc = setProperty( "MaxIterations", 5 );
  Assert( sc.isSuccess(), "Unable to redefine property 'MaxIterations'" );
  sc = setProperty( "MaxIterForAdd", 3 );
  Assert( sc.isSuccess(), "Unable to redefine property 'MaxIterForAdd'" );
  sc = setProperty( "MaxIterForRemove", 3 );
  Assert( sc.isSuccess(), "Unable to redefine property 'MaxIterForRemove'" );
  sc = setProperty( "DeltaDistance", 3 * Gaudi::Units::micrometer );
  Assert( sc.isSuccess(), "Unable to redefine property 'DeltaDistance'" );
  sc = setProperty( "DeltaChi2", 10 * Gaudi::Units::perCent );
  Assert( sc.isSuccess(), "Unable to redefine property 'DeltaChi2'" );
  sc = setProperty( "TransportTolerance", 40 * Gaudi::Units::micrometer );
  Assert( sc.isSuccess(), "Unable to redefine property 'TransportTolerance'" );
  sc = setProperty( "TransportTolerance", 40 * Gaudi::Units::micrometer );
  Assert( sc.isSuccess(), "Unable to redefine property 'TransportTolerance'" );
}
// ============================================================================
/// the factory needed for instantiation
DECLARE_COMPONENT( LoKi::FastVertexFitter )
// ============================================================================

// ============================================================================
// The END
// ============================================================================
