/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// Include files
// ============================================================================
// Event
// ============================================================================
#include "Event/Vertex.h"
// ============================================================================
// LHCbKernel
// ============================================================================
#include "Kernel/InstanceCounter.h"
// ============================================================================
// DaVinciKernel
// ============================================================================
#include "Kernel/IDistanceCalculator.h"
// ============================================================================
// LoKiPhys
// ============================================================================
#include "LoKi/ImpParBase.h"
// ============================================================================
/** @file
 *
 * Implementation file for class LoKi::Vertices::ImpParBase
 *
 *  This file is a part of LoKi project -
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
 *  contributions and advices from G.Raven, J.van Tilburg,
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 *  @date 2006-02-16
 */
// ============================================================================
namespace {
  InstanceCounter<LoKi::Vertices::ImpParBase> s_IPBCounter;
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LHCb::VertexBase* vertex, const IDistanceCalculator* tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( vertex ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Point3D& point, const IDistanceCalculator* tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( point ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Vertices::VertexHolder& holder, const IDistanceCalculator* tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( holder ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LHCb::VertexBase*                     vertex,
                                        const LoKi::Interface<IDistanceCalculator>& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( vertex ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Point3D& point, const LoKi::Interface<IDistanceCalculator>& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( point ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const VertexHolder& holder, const LoKi::Interface<IDistanceCalculator>& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( holder ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LHCb::VertexBase* vertex, const LoKi::Vertices::ImpactParamTool& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( vertex ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Point3D& point, const LoKi::Vertices::ImpactParamTool& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( point ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Vertices::VertexHolder&    holder,
                                        const LoKi::Vertices::ImpactParamTool& tool )
    : LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( holder ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  constructor
// ============================================================================
LoKi::Vertices::ImpParBase::ImpParBase( const LoKi::Vertices::ImpParBase& tool )
    : LoKi::AuxFunBase( tool ), LoKi::Vertices::ImpactParamTool( tool ), LoKi::Vertices::VertexHolder( tool ) {
#ifdef LOKI_DEBUG
  s_IPBCounter.increment();
#endif
}
// ============================================================================
//  destructor
// ============================================================================
LoKi::Vertices::ImpParBase::~ImpParBase() {
#ifdef LOKI_DEBUG
  s_IPBCounter.decrement();
#endif
  //
}
// ============================================================================

// ============================================================================
// The END
// ============================================================================
