/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef PARTICLE2STATE_H
#define PARTICLE2STATE_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTool.h"
#include "Kernel/IParticle2State.h" // Interface

typedef ROOT::Math::SMatrix<double, 2, 5> Matrix2x5;
typedef ROOT::Math::SMatrix<double, 3, 5> Matrix3x5;

/** @class Particle2State Particle2State.h
 *
 *  Convert Particles to State and Back.
 *
 *  @author Patrick Koppenburg
 *  @date   2006-05-16
 */
class Particle2State : public GaudiTool, virtual public IParticle2State {
public:
  /// Standard constructor
  Particle2State( const std::string& type, const std::string& name, const IInterface* parent );

  ~Particle2State(); ///< Destructor

  /// Fill a Particle from a State
  StatusCode state2Particle( const LHCb::State& state, LHCb::Particle& particle ) const override;

  /// Fill a State from Particle
  StatusCode particle2State( const LHCb::Particle& particle, LHCb::State& state ) const override;

  /// test
  StatusCode test( const LHCb::Particle& particle ) const override;

  /// test
  StatusCode test( const LHCb::State& state ) const override;

private:
  /// Get Jacobian for Tx,Ty,q/P -> Px,Py,Pz transformation
  void stateJacobian( const int charge, const Gaudi::XYZVector& Momentum, Gaudi::Matrix5x5& Jacob ) const;
};
#endif // PARTICLE2STATE_H
