/*****************************************************************************\
* (c) Copyright 2000-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/

#include <boost/filesystem.hpp>

#include <map>
#include <string>
#include <vector>

std::map<std::string, std::vector<std::pair<std::string, std::string>>> dataSets;

auto nA = []( const std::string& label, const std::string& value ) {
  return ( value.empty() ? "" : label + "-" + value + "/" );
};

auto add = [&]( const auto& tag, const auto& name, std::string cuslabel = "" ) {
  auto full_name = dir + "/" + name + "/" + fName;
  boost::replace_all( full_name, "//", "/" );
  if ( boost::filesystem::exists( full_name ) ) {
    std::cout << "Adding to '" << tag << "' '" << name << std::endl;
    if ( cuslabel.empty() ) { cuslabel = name; }
    dataSets[tag].push_back( std::make_pair( name, cuslabel ) );
  } else {
    std::cout << "WARNING: '" << full_name << "' does not exist" << std::endl;
  }
};

auto findAll = [&]( const auto& tag, const auto& area, const std::vector<std::string> settings,
                    std::string cuslabel = "" ) {
  boost::filesystem::recursive_directory_iterator end_itr;
  for ( boost::filesystem::recursive_directory_iterator i( area ); i != end_itr; ++i ) {
    if ( i->path().string().find( fName ) != std::string::npos ) {
      if ( std::all_of( settings.begin(), settings.end(),
                        [&i]( const auto& s ) { return i->path().string().find( s ) != std::string::npos; } ) ) {
        auto p = i->path().parent_path().string();
        boost::replace_all( p, dir, "" );
        add( tag, p, cuslabel );
      }
    }
  }
};
