/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef TRACKFORPV_H
#define TRACKFORPV_H 1

// Include files
#include "Event/Track.h"

/** @class TrackForPV TrackForPV.h
 *  Workable track
 *
 *  @author Olivier Callot
 *  @date   2011-11-15
 */
class TrackForPV {
public:
  /// Standard constructor
  TrackForPV( LHCb::Track* track, double xBeam, double yBeam );

  virtual ~TrackForPV(); ///< Destructor

  double zAtBeam() const { return m_zAtBeam; }

  double rAtBeam() const { return m_rAtBeam; }

  LHCb::Track* track() { return m_track; }

  double tx() { return m_dir.x(); }
  double ty() { return m_dir.y(); }
  double x0() { return m_point.x() - m_dir.x() * m_point.z(); }
  double y0() { return m_point.y() - m_dir.y() * m_point.z(); }
  double wx() { return m_wx; }
  double wy() { return m_wy; }

  double chi2( Gaudi::XYZPoint& pt ) {
    double dx = m_point.x() + m_dir.x() * ( pt.z() - m_point.z() ) - pt.x();
    double dy = m_point.y() + m_dir.y() * ( pt.z() - m_point.z() ) - pt.y();
    return m_wx * dx * dx + m_wy * dy * dy;
  }

  struct LowerByZ {
    bool operator()( const TrackForPV* lhs, const TrackForPV* rhs ) const { return lhs->zAtBeam() < rhs->zAtBeam(); }
  };

  bool used() { return m_used; }
  void setUsed( bool flag ) { m_used = flag; }

protected:
private:
  LHCb::Track*     m_track;
  Gaudi::XYZPoint  m_point;
  Gaudi::XYZVector m_dir;
  double           m_wx;
  double           m_wy;
  double           m_zAtBeam;
  double           m_rAtBeam;
  bool             m_used;
};
#endif // TRACKFORPV_H
